"""
Cortex Agent API.

OpenAPI 3.0 specification for the Cortex Agent API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictInt


class AgentOrchestrationBudget(BaseModel):
    """A model object representing the AgentOrchestrationBudget resource.

    Constructs an object of type AgentOrchestrationBudget with the provided properties.

    Parameters
    __________
    seconds : int, optional
        Time budget in seconds.
    tokens : int, optional
        Token budget.
    """

    seconds: Optional[StrictInt] = None

    tokens: Optional[StrictInt] = None

    __properties = ["seconds", "tokens"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AgentOrchestrationBudget:
        """Create an instance of AgentOrchestrationBudget from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AgentOrchestrationBudget:
        """Create an instance of AgentOrchestrationBudget from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AgentOrchestrationBudget.model_validate(obj)

        _obj = AgentOrchestrationBudget.model_validate(
            {
                "seconds": obj.get("seconds"),
                "tokens": obj.get("tokens"),
            }
        )

        return _obj


class AgentOrchestrationBudgetModel:
    def __init__(
        self,  # optional properties
        seconds: Optional[int] = None,
        tokens: Optional[int] = None,
    ):
        """A model object representing the AgentOrchestrationBudget resource.

        Constructs an object of type AgentOrchestrationBudget with the provided properties.

        Parameters
        __________
        seconds : int, optional
            Time budget in seconds.

        tokens : int, optional
            Token budget.
        """
        self.seconds = seconds
        self.tokens = tokens

    __properties = ["seconds", "tokens"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AgentOrchestrationBudget(
            seconds=self.seconds,
            tokens=self.tokens,
        )

    @classmethod
    def _from_model(cls, model) -> AgentOrchestrationBudgetModel:
        return AgentOrchestrationBudgetModel(
            seconds=model.seconds,
            tokens=model.tokens,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a AgentOrchestrationBudget.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AgentOrchestrationBudget object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AgentOrchestrationBudgetModel:
        """Create an instance of AgentOrchestrationBudget from a dict.

        This method constructs a AgentOrchestrationBudget object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AgentOrchestrationBudget
            A AgentOrchestrationBudget object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AgentOrchestrationBudget.from_dict(obj))


AgentOrchestrationBudget._model_class = AgentOrchestrationBudgetModel



import pytest

from snowflake.core.alert import Alert, MinutesSchedule
from snowflake.core.schema import Schema
from tests.utils import random_string

from ...utils import ensure_snowflake_version


@pytest.fixture
def temp_alert(alerts, snowflake_version):
    ensure_snowflake_version(snowflake_version, "8.36.0")

    alert_name = random_string(10, "test_alert_")
    alert_handle = alerts.create(
        Alert(
            name=alert_name,
            condition="SELECT 1",
            action="SELECT 2",
            schedule=MinutesSchedule(minutes=1),
            comment="ThIs iS a ComMeNT",
        )
    )

    try:
        yield alert_handle
    finally:
        alerts[alert_name].drop()


def test_clone(alerts, temp_alert):
    alert_name = random_string(10, "test_alert_clone_")

    cloned_alert = alerts.create(alert_name, clone_alert=temp_alert.name)

    try:
        cloned_handle = cloned_alert.fetch()
        assert cloned_handle.name.upper() == alert_name.upper()
        assert cloned_handle.condition == "SELECT 1"
        assert cloned_handle.action == "SELECT 2"
        assert cloned_handle.schedule.minutes == 1
        assert cloned_handle.comment == "ThIs iS a ComMeNT"
    finally:
        alerts[alert_name].drop()


def test_clone_across_schema(temp_alert, temp_schema):
    alert_name = random_string(10, "test_clone_alert_across_schema_")

    created_handle = temp_schema.alerts.create(
        alert_name, clone_alert=f"{temp_alert.schema.name}.{temp_alert.name}"
    ).fetch()

    try:
        assert created_handle.database_name.upper() == temp_schema.database.name.upper()
        assert created_handle.schema_name.upper() == temp_schema.name.upper()
        assert created_handle.comment == "ThIs iS a ComMeNT"
        assert created_handle.name.upper() == alert_name.upper()
        assert created_handle.condition == "SELECT 1"
        assert created_handle.action == "SELECT 2"
        assert created_handle.schedule.minutes == 1
    finally:
        temp_schema.alerts[alert_name].drop()


def test_clone_across_database(temp_alert, temp_db):
    schema_name = random_string(10, "test_create_clone_across_database_schema_name_")
    created_schema = temp_db.schemas.create(Schema(name=schema_name))
    alert_name = random_string(10, "test_alert_clone_across_database_")

    try:
        created_handle = created_schema.alerts.create(
            alert_name, clone_alert=f"{temp_alert.database.name}.{temp_alert.schema.name}.{temp_alert.name}"
        ).fetch()

        assert created_handle.database_name.upper() == created_schema.database.name.upper()
        assert created_handle.schema_name.upper() == created_schema.name.upper()
        assert created_handle.comment == "ThIs iS a ComMeNT"
        assert created_handle.name.upper() == alert_name.upper()
        assert created_handle.condition == "SELECT 1"
        assert created_handle.action == "SELECT 2"
        assert created_handle.schedule.minutes == 1
    finally:
        created_schema.drop()



from typing import TYPE_CHECKING

from snowflake.core.external_volume._generated.api.external_volume_api_base import (
    ExternalVolumeCollectionBase,
    ExternalVolumeResourceBase,
)


if TYPE_CHECKING:
    from snowflake.core import Root


class ExternalVolumeCollection(ExternalVolumeCollectionBase):
    """Represents the collection operations of the Snowflake External Volume resource.

    With this collection, you can create, iterate through, and search for external volume that you have access to
    in the current context.

    Examples
    ________
    Creating an external volume instance:

    >>> external_volume_collection = root.external_volumes
    >>> external_volume = ExternalVolume(
    ...     name="MY_EXTERNAL_VOLUME",
    ...     storage_location=StorageLocationS3(
    ...         name="abcd-my-s3-us-west-2",
    ...         storage_base_url="s3://MY_EXAMPLE_BUCKET/",
    ...         storage_aws_role_arn="arn:aws:iam::123456789022:role/myrole",
    ...         encryption=Encryption(
    ...             type="AWS_SSE_KMS", kms_key_id="1234abcd-12ab-34cd-56ef-1234567890ab"
    ...         ),
    ...     ),
    ...     comment="This is my external volume",
    ... )
    >>> external_volume_collection.create(external_volume)
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, ExternalVolumeResource)


class ExternalVolumeResource(ExternalVolumeResourceBase):
    """Represents a reference to a Snowflake external volume.

    With this external volume reference, you can fetch information about an external volume, as well
    as perform certain actions on it.
    """

    _plural_name = "external_volumes"



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.


import pytest

from snowflake.core._common import CreateMode
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_fetch(warehouses):
    warehouse_name = random_string(5, "test_create_warehouse_")
    test_warehouse = Warehouse(
        name=warehouse_name,
        warehouse_size="SMALL",
        auto_suspend=500,
        warehouse_type="STANDARD",
        auto_resume="false",
        initially_suspended="true",
        comment="This IS a COmment",
        max_concurrency_level=2,
        statement_queued_timeout_in_seconds=4,
        statement_timeout_in_seconds=2,
    )

    warehouse_ref = None
    try:
        # Test warehouse create.
        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.error_if_exists)
        warehouse = warehouse_ref.fetch()
        assert warehouse_name.upper() == warehouse.name.upper()
        assert warehouse.size.upper() == "SMALL"
        assert warehouse.auto_suspend == 500
        assert warehouse.type.startswith("STANDARD")
        assert warehouse.auto_resume.upper() == "FALSE"
        # assert warehouse.initially_suspended == "TRUE"
        assert warehouse.comment == "This IS a COmment"
        assert warehouse.max_concurrency_level == 2
        assert warehouse.statement_queued_timeout_in_seconds == 4
        assert warehouse.statement_timeout_in_seconds == 2
        # assert warehouse.tags=="{\"additionalProp1\":\"hello\"}"
    finally:
        warehouse_ref.drop()



import copy

import pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_password_policy_template


def test_rename(password_policies):
    original_name = random_string(10, "test_original_password_policy_")
    new_name = random_string(10, "test_renamed_password_policy_")

    password_policy = copy.deepcopy(test_password_policy_template)
    password_policy.name = original_name
    password_policy_handle = password_policies.create(password_policy)

    try:
        fetched_policy = password_policy_handle.fetch()
        assert fetched_policy.name.upper() == original_name.upper()

        password_policy_handle.rename(new_name)

        fetched_policy = password_policy_handle.fetch()
        assert fetched_policy.name == new_name.upper()
        assert fetched_policy.schema_name == password_policies.schema.name.upper()
        assert fetched_policy.database_name == password_policies.database.name.upper()

        with pytest.raises(NotFoundError):
            password_policies[original_name].fetch()

    finally:
        password_policy_handle.drop(if_exists=True)


def test_rename_nonexistent_password_policy(password_policies):
    """Test renaming a non-existent password policy raises NotFoundError."""
    nonexistent_name = random_string(10, "test_nonexistent_password_policy_")
    new_name = random_string(10, "test_new_name_password_policy_")

    with pytest.raises(NotFoundError):
        password_policies[nonexistent_name].rename(new_name)

    password_policies[nonexistent_name].rename(new_name, if_exists=True)


def test_rename_password_policy_cross_schema(password_policies, temp_schema):
    """Test renaming password policy across schemas."""
    original_name = random_string(10, "test_cross_schema_password_policy_")
    new_name = random_string(10, "test_renamed_cross_schema_password_policy_")

    password_policy = copy.deepcopy(test_password_policy_template)
    password_policy.name = original_name
    password_policy_handle = password_policies.create(password_policy)

    try:
        password_policy_handle.rename(
            new_name, target_schema=temp_schema.name, target_database=password_policy_handle.database.name
        )

        fetched_policy = password_policy_handle.fetch()
        assert fetched_policy.name.upper() == new_name.upper()
        assert fetched_policy.schema_name.upper() == temp_schema.name.upper()
        assert fetched_policy.database_name.upper() == password_policy_handle.database.name.upper()

        with pytest.raises(NotFoundError):
            password_policies[original_name].fetch()

    finally:
        password_policy_handle.drop(if_exists=True)



import pytest

from pydantic import ValidationError

from tests.integ.dynamic_table.util import assert_dynamic_table


@pytest.mark.min_sf_ver("8.27.0")
def test_iter_like(dynamic_tables, dynamic_table_handle, table_handle, database, schema, db_parameters):
    listed_tables_deep = list(dynamic_tables.iter(like=dynamic_table_handle.name, deep=True))
    assert_dynamic_table(
        listed_tables_deep[0], dynamic_table_handle.name, table_handle, database, schema, db_parameters, True
    )

    listed_tables_not_deep = list(dynamic_tables.iter(like=dynamic_table_handle.name, deep=False))
    assert_dynamic_table(
        listed_tables_not_deep[0], dynamic_table_handle.name, table_handle, database, schema, db_parameters, False
    )


@pytest.mark.min_sf_ver("8.27.0")
def test_iter_starts_with(dynamic_tables, dynamic_table_handle, table_handle, database, schema, db_parameters):
    listed_tables_deep = list(dynamic_tables.iter(starts_with=dynamic_table_handle.name.upper(), deep=True))
    assert len(listed_tables_deep) == 1
    assert_dynamic_table(
        listed_tables_deep[0], dynamic_table_handle.name, table_handle, database, schema, db_parameters, True
    )

    listed_tables_deep = list(dynamic_tables.iter(starts_with="zzz", deep=True))
    assert len(listed_tables_deep) == 0


def test_iter_limit(dynamic_tables, dynamic_table_handle):
    data = list(dynamic_tables.iter(limit=10))
    assert 0 < len(data) <= 10
    assert len(list(dynamic_tables.iter(limit=10, from_name="zzzzzzzzzzzz"))) == 0

    with pytest.raises(ValidationError):
        data = list(dynamic_tables.iter(limit=10001))



import subprocess
import time

import pytest


"""
Since the macos testing image doesn't have docker installed
We will create the image repo manually

run this by
```
pytest setup_manually.py --no-cov -s
```

!!! Make sure you update 'imagerepo' function in conftest.py !!!
"""

DATABASE = "TESTDB_PYTHON_AUTO"
SCHEMA = "TESTSCHEMA_AUTO"
AUTO_image_repo = "TEST_IMAGE_REPO_AUTO"


def test_setup_image(connection, db_parameters, setup_basic):
    with connection.cursor() as cursor:
        cursor.execute(f"USE DATABASE {DATABASE}")
        cursor.execute(f"CREATE SCHEMA IF NOT EXISTS {SCHEMA}")
        cursor.execute(f"USE SCHEMA {SCHEMA}")

        check_docker_h_output = subprocess.getoutput("docker --help")
        if len(check_docker_h_output) < 50 or "command not found" in check_docker_h_output:
            raise Exception("Docker not installed")

        subprocess.run("docker pull --platform amd64 hello-world".split(" "))
        subprocess.run("docker pull --platform amd64 nginx".split(" "))

        cursor.execute(f"CREATE IMAGE REPOSITORY IF NOT EXISTS {AUTO_image_repo};")
        repo_url = cursor.execute(f"SHOW IMAGE REPOSITORIES LIKE '{AUTO_image_repo}';").fetchone()[4]
        host_name, _, _ = repo_url.partition("/")

        # Be careful with failures here not to print tokens.
        #  To avoid printing local variables use pytest.fail
        try:
            completed_process = subprocess.run(
                f"docker login {host_name} -u {db_parameters['user']} --password-stdin".split(" "),
                input=db_parameters["password"].encode() + b"\n",
                capture_output=True,
            )
            assert completed_process.returncode == 0
        except Exception:
            pytest.fail("Docker login failure")

        for image_name in ["hello-world", "nginx"]:
            subprocess.run(f"docker image tag {image_name} {repo_url}/{image_name}:latest".split(" "))
            upload_succ = False
            for _ in range(3):
                completed_process = subprocess.run(f"docker push {repo_url}/{image_name}:latest".split(" "))
                if completed_process.returncode != 0:
                    time.sleep(1)
                    continue
                upload_succ = True
                break
            assert upload_succ

        print("!!!!! Update Repo Url !!!!!")
        print(repo_url)



def clear_default_grant(grants):
    new_grants = []
    for grant in grants:
        if not (grant.securable_type == "DATABASE" and grant.privileges == ["USAGE"]):
            new_grants.append(grant)
    return new_grants


def assert_basic_grant(grant, check_granted_by=True):
    if check_granted_by:
        assert grant.granted_by is not None
    assert grant.created_on is not None
    assert grant.containing_scope is None
    assert grant.privileges is not None
    assert grant.securable is not None



from snowflake.core.function import FunctionArgument, ServiceFunction


def create_service_function(name, arg_types, returns, endpoint, temp_service, functions):
    return functions.create(
        ServiceFunction(
            name=name,
            arguments=[FunctionArgument(name=f"v_{str(i)}", datatype=v) for i, v in enumerate(arg_types)],
            returns=returns,
            service=temp_service,
            endpoint=endpoint,
            path="/path/to/myapp",
            max_batch_rows=5,
        )
    )



"""
Snowflake User API.

The Snowflake User API is a REST API that you can use to access, update, and perform certain action on Users in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, field_validator
from typing_extensions import Annotated


class ContainingScope(BaseModel):
    """A model object representing the ContainingScope resource.

    Constructs an object of type ContainingScope with the provided properties.

    Parameters
    __________
    database : str
        Database name of the securable scope if applicable.
    var_schema : str, optional
        Schema name of the securable scope if applicable.
    """

    database: Annotated[str, Field(strict=True)]

    var_schema: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, alias="schema")

    __properties = ["database", "schema"]

    @field_validator("database")
    def database_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("var_schema")
    def var_schema_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ContainingScope:
        """Create an instance of ContainingScope from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ContainingScope:
        """Create an instance of ContainingScope from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ContainingScope.model_validate(obj)

        _obj = ContainingScope.model_validate(
            {
                "database": obj.get("database"),
                "var_schema": obj.get("schema"),
            }
        )

        return _obj


class ContainingScopeModel:
    def __init__(
        self,
        database: str,
        # optional properties
        var_schema: Optional[str] = None,
    ):
        """A model object representing the ContainingScope resource.

        Constructs an object of type ContainingScope with the provided properties.

        Parameters
        __________
        database : str
            Database name of the securable scope if applicable.
        var_schema : str, optional
            Schema name of the securable scope if applicable.
        """
        self.database = database
        self.var_schema = var_schema

    __properties = ["database", "schema"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ContainingScope(
            database=self.database,
            var_schema=self.var_schema,
        )

    @classmethod
    def _from_model(cls, model) -> ContainingScopeModel:
        return ContainingScopeModel(
            database=model.database,
            var_schema=model.var_schema,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ContainingScope.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ContainingScope object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ContainingScopeModel:
        """Create an instance of ContainingScope from a dict.

        This method constructs a ContainingScope object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ContainingScope
            A ContainingScope object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ContainingScope.from_dict(obj))


ContainingScope._model_class = ContainingScopeModel



import pytest as pytest

from snowflake.core.network_policy import NetworkPolicy
from tests.integ.utils import random_string


@pytest.fixture(scope="session")
def prepare_rules_for_network_polices(cursor):
    allowed_rules = []
    blocked_rules = []
    for _ in range(3):
        allow_rule_name = random_string(10, "allowed_network_rule_").upper()
        cursor.execute(f"""
            CREATE NETWORK RULE IF NOT EXISTS {allow_rule_name}
                MODE=INTERNAL_STAGE TYPE=AWSVPCEID VALUE_LIST=('vpce-1234567{_ % 10}');
                    """).fetchone()
        allowed_rules.append(allow_rule_name)

    for _ in range(5):
        blocked_rule_name = random_string(10, "blocked_network_rule_").upper()
        cursor.execute(f"""
            CREATE NETWORK RULE IF NOT EXISTS {blocked_rule_name}
                MODE=INTERNAL_STAGE TYPE=AWSVPCEID VALUE_LIST=('vpce-1234567{_ % 10}');
                    """).fetchone()
        blocked_rules.append(blocked_rule_name)

    try:
        yield allowed_rules, blocked_rules
    finally:
        for r in allowed_rules + blocked_rules:
            cursor.execute(f"DROP NETWORK RULE IF EXISTS {r}")


@pytest.fixture
def template_network_policy(prepare_rules_for_network_polices):
    allowed_rules, blocked_rules = prepare_rules_for_network_polices

    return NetworkPolicy(
        name="to_be_named",
        allowed_network_rule_list=allowed_rules,
        blocked_network_rule_list=blocked_rules,
        allowed_ip_list=["8.8.8.8"],
        blocked_ip_list=["0.0.0.0"],
    )



from typing import TYPE_CHECKING

from snowflake.core.pipe._generated.api.pipe_api_base import PipeCollectionBase, PipeResourceBase


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class PipeCollection(PipeCollectionBase):
    """Represents the collection operations of the Snowflake Pipe resource.

    With this collection, you can create, iterate through, and search for pipes that you have access to
    in the current context.

    Examples
    ________
    Creaing a pipe instance:

    >>> pipes = root.databases["my_db"].schemas["my_schema"].pipes
    >>>     new_pipe = Pipe(
    ...         name="my_pipe",
    ...         comment="This is a pipe")
    >>> pipes.create(new_pipe)
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, PipeResource)


class PipeResource(PipeResourceBase):
    """Represents a reference to a Snowflake pipe.

    With this pipe reference, you can fetch information about a pipe, as well as perform certain
    actions on it.
    """

    _plural_name = "pipes"



from ._generated import CompleteRequestMessagesInner, CortexInferenceApi, StreamingTextContent
from ._inference_service import CompleteRequest, CortexInferenceService


CortexInferenceServiceApi = CortexInferenceApi

__all__ = [
    "CortexInferenceService",
    "CompleteRequest",
    "CompleteRequestMessagesInner",
    "CortexInferenceServiceApi",
    "StreamingTextContent",
]



import pytest

from snowflake.core.streamlit import Streamlit
from tests.integ.utils import random_string


pytestmark = [pytest.mark.min_sf_ver("9.38.0")]


@pytest.fixture(scope="module")
def streamlits_extended(streamlits, streamlit_stage_with_file, warehouse, streamlit_main_file):
    names_list = []

    for _ in range(5):
        names_list.append(random_string(10, "test_streamlit_iter_a_"))

    for _ in range(7):
        names_list.append(random_string(10, "test_streamlit_iter_b_"))

    for _ in range(3):
        names_list.append(random_string(10, "test_streamlit_iter_c_"))

    try:
        for name in names_list:
            st = Streamlit(
                name=name,
                query_warehouse=warehouse.name,
                source_location=f"@{streamlit_stage_with_file.name}",
                main_file=streamlit_main_file,
            )
            streamlits.create(st)

        yield streamlits
    finally:
        for name in names_list:
            streamlits[name].drop(if_exists=True)


def test_iter_raw(streamlits_extended):
    assert len(list(streamlits_extended.iter())) >= 15


def test_iter_like(streamlits_extended):
    assert len(list(streamlits_extended.iter(like="test_streamlit_iter_a_%"))) == 5
    assert len(list(streamlits_extended.iter(like="test_streamlit_iter_b_%"))) == 7
    assert len(list(streamlits_extended.iter(like="test_streamlit_iter_c_%"))) == 3
    assert len(list(streamlits_extended.iter(like="TEST_STREAMLIT_ITER_C_%"))) == 3
    assert len(list(streamlits_extended.iter(like="nonexistent_pattern_%"))) == 0


def test_iter_limit(streamlits_extended):
    assert len(list(streamlits_extended.iter(limit=2))) == 2


def test_iter_starts_with(streamlits_extended):
    assert len(list(streamlits_extended.iter(starts_with="test_streamlit_iter_a_".upper()))) == 5
    assert len(list(streamlits_extended.iter(starts_with="test_streamlit_iter_a_"))) == 0
    assert len(list(streamlits_extended.iter(starts_with="test_streamlit_iter_d_".upper()))) == 0


def test_iter_from_name(streamlits_extended):
    streamlits_from_b = list(streamlits_extended.iter(limit=20, from_name="TEST_STREAMLIT_ITER_B_"))
    assert len(streamlits_from_b) == 10



import typing

import pytest

from snowflake.core.exceptions import InvalidOperationError
from snowflake.core.task import StoredProcedureCall
from snowflake.core.task.dagv1 import DAG, DAGTask


if typing.TYPE_CHECKING:
    from snowflake.snowpark import Session


def foo1(session: "Session") -> str:
    return "abc"


def foo2(session: "Session") -> str:
    return "abc"


def foo3(session: "Session") -> str:
    return "abc"


def foo4(session: "Session") -> str:
    return "abc"


@pytest.mark.snowpark
def test__use_func_return_value_kicked_in():
    with DAG("dag1", stage_location="fake_stage", use_func_return_value=True):
        task1 = DAGTask("task1", foo1)

    with DAG("dag2", stage_location="fake_stage"):
        task2 = DAGTask("task2", foo2)

    with DAG("dag3", use_func_return_value=True):
        task3 = DAGTask("task3", StoredProcedureCall(foo3, stage_location="fake_stage"))

    with DAG("dag4"):
        task4 = DAGTask("task4", StoredProcedureCall(foo4, stage_location="fake_stage"))

    lower_task1 = task1._to_low_level_task()
    lower_task2 = task2._to_low_level_task()
    lower_task3 = task3._to_low_level_task()
    lower_task4 = task4._to_low_level_task()
    assert lower_task1.definition.func is not foo1
    assert lower_task2.definition.func is foo2
    assert lower_task3.definition.func is not foo3
    assert lower_task4.definition.func is foo4


def test_create_task_in_dag_without_warehouse():
    dag_warehouse = "DAG_FAKE_WAREHOUSE"
    with DAG("dag", stage_location="fake_stage", warehouse=dag_warehouse):
        task = DAGTask("task", "select 'task'")
    assert task.warehouse == dag_warehouse


def test_create_task_in_dag_with_warehouse():
    dag_warehouse = "DAG_FAKE_WAREHOUSE"
    task_warehouse = "TASK_FAKE_WAREHOUSE"
    with DAG("dag", stage_location="fake_stage", warehouse=dag_warehouse):
        task = DAGTask("task", "select 'task'", warehouse=task_warehouse)
    assert task.warehouse == task_warehouse


def test_skip_adding_predecessors_if_task_is_from_different_task_group(schema):
    with DAG("dag1"):
        task1 = DAGTask("task1", "select 1")
        task2 = DAGTask("task2", "select 2")

    with DAG("dag2"):
        task3 = DAGTask("task3", "select 3")

    with pytest.raises(InvalidOperationError) as err:
        task1.add_predecessors(task3)
    assert str(err.value) == f"Task {task3.name} belongs to a different task graph"
    assert task1.predecessors == set()

    with pytest.raises(InvalidOperationError) as err:
        task1.add_predecessors([task2, task3])
    assert str(err.value) == f"Task {task3.name} belongs to a different task graph"
    assert task1.predecessors == set()


def test_skip_adding_successors_if_task_is_from_different_task_group(schema):
    with DAG("dag1"):
        task1 = DAGTask("task1", "select 1")
        task2 = DAGTask("task2", "select 2")

    with DAG("dag2"):
        task3 = DAGTask("task3", "select 3")

    with pytest.raises(InvalidOperationError) as err:
        task1.add_successors(task3)
    assert str(err.value) == f"Task {task3.name} belongs to a different task graph"
    assert task3.predecessors == set()

    with pytest.raises(InvalidOperationError) as err:
        task1.add_successors([task2, task3])
    assert str(err.value) == f"Task {task3.name} belongs to a different task graph"
    assert task2.predecessors == set()
    assert task3.predecessors == set()


def test_add_predecessor_to_finalizer_and_vice_versa(schema):
    with DAG("dag"):
        task1 = DAGTask("task1", "select 1", is_finalizer=True)
        task2 = DAGTask("task2", "select 1")
        task3 = DAGTask("task3", "select 1")

        with pytest.raises(InvalidOperationError) as err:
            task1.add_predecessors(task2)
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any predecessors"
        assert task1.predecessors == set()

        with pytest.raises(InvalidOperationError) as err:
            task2.add_predecessors([task3, task1])
        assert str(err.value) == f"Finalizer task {task1.name} cannot be predecessor of any task"
        assert task2.predecessors == set()

        with pytest.raises(InvalidOperationError) as err:
            task2 << task1
        assert str(err.value) == f"Finalizer task {task1.name} cannot be predecessor of any task"
        assert task2.predecessors == set()

        with pytest.raises(InvalidOperationError) as err:
            task1 << task2
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any predecessors"
        assert task1.predecessors == set()


def test_add_successors_to_finalizer_and_vice_versa(schema):
    with DAG("dag"):
        task1 = DAGTask("task1", "select 1", is_finalizer=True)
        task2 = DAGTask("task2", "select 1")
        task3 = DAGTask("task3", "select 1")

        with pytest.raises(InvalidOperationError) as err:
            task1.add_successors(task2)
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any successors"
        assert task2.predecessors == set()

        with pytest.raises(InvalidOperationError) as err:
            task2.add_successors(task1)
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any predecessors"
        assert task1.predecessors == set()

        with pytest.raises(InvalidOperationError) as err:
            task2.add_successors([task3, task1])
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any predecessors"
        assert task1.predecessors == set()
        assert task3.predecessors == set()

        # adding finalizer task1 as successor of task2
        with pytest.raises(InvalidOperationError) as err:
            task2 >> task1
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any predecessors"
        assert task1.predecessors == set()

        # adding task2 as the successor of finalizer task1
        with pytest.raises(InvalidOperationError) as err:
            task1 >> task2
        assert str(err.value) == f"Finalizer task {task1.name} cannot have any successors"
        assert task2.predecessors == set()



import json

from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.exceptions import NotFoundError
from snowflake.core.iceberg_table._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def iceberg_tables(schema):
    return schema.iceberg_tables


@pytest.fixture
def iceberg_table(iceberg_tables):
    return iceberg_tables["my_table"]


def test_set_tags(fake_root, iceberg_table, tag, tags):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/iceberg-tables/my_table:set-tags",
    )
    tag_assignments = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tag_assignments.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        iceberg_table.set_tags(tag_assignments)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = iceberg_table.set_tags_async(tag_assignments)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)

    # Check if `NotFoundError` will be propagated.
    missing_tag = tags["does_not_exist"]
    missing_kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value="value",
                tag_name=missing_tag.name,
                tag_schema=missing_tag.schema.name,
                tag_database=missing_tag.database.name,
            ).to_dict()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST, side_effect=NotFoundError("not found")) as mocked_request:
        with pytest.raises(NotFoundError):
            iceberg_table.set_tags({missing_tag: TagValue(value="value")})
    mocked_request.assert_called_once_with(*args, **missing_kwargs)


@pytest.mark.parametrize("if_exists", [True, False])
def test_set_tags_with_query_parameters(fake_root, iceberg_table, tag, if_exists):
    args = (
        fake_root,
        "POST",
        BASE_URL + f"/databases/my_db/schemas/my_schema/iceberg-tables/my_table:set-tags?ifExists={if_exists}",
    )
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        query_params=[("ifExists", if_exists)],
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ],
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        iceberg_table.set_tags(tags, if_exists=if_exists)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = iceberg_table.set_tags_async(tags, if_exists=if_exists)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tag(fake_root, iceberg_table, tag, tags):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/iceberg-tables/my_table:unset-tags",
    )
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        iceberg_table.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = iceberg_table.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)

    # Check if `NotFoundError` will be propagated.
    missing_tag = tags["does_not_exist"]
    missing_kwargs = extra_params(
        body=[
            TagReference(
                tag_name=missing_tag.name,
                tag_schema=missing_tag.schema.name,
                tag_database=missing_tag.database.name,
            ).to_dict()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST, side_effect=NotFoundError("not found")) as mocked_request:
        with pytest.raises(NotFoundError):
            iceberg_table.unset_tags({missing_tag})
    mocked_request.assert_called_once_with(*args, **missing_kwargs)


@pytest.mark.parametrize("if_exists", [True, False])
def test_unset_tags_with_query_parameters(fake_root, iceberg_table, tag, if_exists):
    args = (
        fake_root,
        "POST",
        BASE_URL + f"/databases/my_db/schemas/my_schema/iceberg-tables/my_table:unset-tags?ifExists={if_exists}",
    )
    tag_resources = {tag}
    kwargs = extra_params(
        query_params=[("ifExists", if_exists)],
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ],
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        iceberg_table.unset_tags(tag_resources, if_exists=if_exists)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = iceberg_table.unset_tags_async(tag_resources, if_exists=if_exists)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, iceberg_table, tag):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/iceberg-tables/my_table:get-tags",
    )
    kwargs = extra_params()
    fake_root.databases = tag.database.collection
    tag_assignment = TagAssignment(
        tag_value="value",
        tag_name=tag.name,
        tag_schema=tag.schema.name,
        tag_database=tag.database.name,
        level="TABLE",
    )
    response = mock_http_response(json.dumps([tag_assignment.to_dict()]))

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = response
        assert iceberg_table.get_tags() == {tag: TagValue("value", "TABLE")}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = response
        op = iceberg_table.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {tag: TagValue("value", "TABLE")}
    mocked_request.assert_called_once_with(*args, **kwargs)

    # Check if `NotFoundError` will be propagated.
    with mock.patch(API_CLIENT_REQUEST, side_effect=NotFoundError("not found")) as mocked_request:
        with pytest.raises(NotFoundError):
            iceberg_table.get_tags()
    mocked_request.assert_called_once_with(*args, **kwargs)


@pytest.mark.parametrize("with_lineage", [False, True])
def test_get_tags_with_lineage_query_params(fake_root, iceberg_table, with_lineage):
    args = (
        fake_root,
        "GET",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/iceberg-tables/my_table:get-tags"
        + f"?withLineage={with_lineage}",
    )
    kwargs = extra_params(query_params=[("withLineage", with_lineage)])
    response = mock_http_response(json.dumps([]))

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = response
        iceberg_table.get_tags(with_lineage=with_lineage)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = response
        op = iceberg_table.get_tags_async(with_lineage=with_lineage)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.dynamic_table import DownstreamLag, DynamicTable, DynamicTableColumn, DynamicTableResource
from snowflake.core.dynamic_table._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


@pytest.fixture
def dynamic_tables(schema):
    return schema.dynamic_tables


@pytest.fixture
def dynamic_table(dynamic_tables):
    return dynamic_tables["my_table"]


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
DYNAMIC_TABLE = DynamicTable(
    name="my_table",
    target_lag=DownstreamLag(),
    warehouse="wh",
    columns=[DynamicTableColumn(name="c1", datatype="int")],
    query="SELECT * FROM foo",
)


def test_create_dynamic_table(fake_root, dynamic_tables):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables?createMode=errorIfExists")
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists")],
        body={
            "name": "my_table",
            "kind": "PERMANENT",
            "target_lag": {"type": "DOWNSTREAM"},
            "warehouse": "wh",
            "columns": [{"name": "c1", "datatype": "int"}],
            "query": "SELECT * FROM foo",
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dt_res = dynamic_tables.create(DYNAMIC_TABLE)
        assert isinstance(dt_res, DynamicTableResource)
        assert dt_res.name == "my_table"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_tables.create_async(DYNAMIC_TABLE)
        assert isinstance(op, PollingOperation)
        dt_res = op.result()
        assert dt_res.name == "my_table"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_dynamic_table_clone(fake_root, dynamic_tables):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/dynamic-tables/temp_clone_table:clone?"
        + "createMode=errorIfExists&copyGrants=False&targetDatabase=my_db&targetSchema=my_schema",
    )
    kwargs = extra_params(
        query_params=[
            ("createMode", "errorIfExists"),
            ("copyGrants", False),
            ("targetDatabase", "my_db"),
            ("targetSchema", "my_schema"),
        ],
        body={"name": "my_table"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_tables.create("my_table", clone_table="temp_clone_table")
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_tables.create_async("my_table", clone_table="temp_clone_table")
        assert isinstance(op, PollingOperation)
        dt_res = op.result()
        assert dt_res.name == "my_table"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_dynamic_table(fake_root, dynamic_tables):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        dynamic_tables.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = dynamic_tables.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_dynamic_table(fake_root, dynamic_table):
    from snowflake.core.dynamic_table._generated.models import DynamicTable as DynamicTableModel

    model = DynamicTableModel(
        name="my_table",
        target_lag=DownstreamLag(),
        warehouse="wh",
        columns=[DynamicTableColumn(name="c1", datatype="int")],
        query="SELECT * FROM foo",
    )

    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        dynamic_table.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = dynamic_table.fetch_async()
        assert isinstance(op, PollingOperation)
        tab = op.result()
        assert tab.to_dict() == DYNAMIC_TABLE.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_dynamic_table(fake_root, dynamic_table):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_undrop_dynamic_table(fake_root, dynamic_table):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:undrop")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.undrop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.undrop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_suspend_dynamic_table(fake_root, dynamic_table):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:suspend")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.suspend()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.suspend_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_resume_dynamic_table(fake_root, dynamic_table):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:resume")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.resume()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.resume_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_refresh_dynamic_table(fake_root, dynamic_table):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:refresh")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.refresh()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.refresh_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_swap_with_dynamic_table(fake_root, dynamic_table):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:swap-with"
        + "?targetName=other_db.other_schema.other_table",
    )
    kwargs = extra_params(query_params=[("targetName", "other_db.other_schema.other_table")])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.swap_with("other_db.other_schema.other_table")
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.swap_with_async("other_db.other_schema.other_table")
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_suspend_recluster_dynamic_table(fake_root, dynamic_table):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:suspend-recluster",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.suspend_recluster()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.suspend_recluster_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_resume_recluster_dynamic_table(fake_root, dynamic_table):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:resume-recluster")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.resume_recluster()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.resume_recluster_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, dynamic_table, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, dynamic_table, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        dynamic_table.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = dynamic_table.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, dynamic_table):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/dynamic-tables/my_table:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert dynamic_table.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = dynamic_table.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



from abc import ABC, abstractmethod
from collections.abc import ItemsView, Iterator, KeysView, ValuesView
from typing import TYPE_CHECKING, Generic, Protocol, TypeVar

from snowflake.connector import SnowflakeConnection


T = TypeVar("T")

if TYPE_CHECKING:
    from snowflake.core import Root


class ObjectCollectionBC(ABC):
    @property
    @abstractmethod
    def _connection(self) -> SnowflakeConnection: ...

    @property
    @abstractmethod
    def root(self) -> "Root": ...


class ObjectCollection(ObjectCollectionBC, Generic[T]):
    def __init__(self, ref_class: type[T]) -> None:
        self._items: dict[str, T] = {}
        self._ref_class = ref_class

    def __getitem__(self, item: str) -> T:
        if item not in self._items:
            # Protocol doesn't support restricting __init__
            self._items[item] = self._ref_class(item, self)  # type: ignore[call-arg]
        return self._items[item]

    def __iter__(self) -> Iterator[str]:
        return iter(self.keys())

    def keys(self) -> KeysView[str]:
        return self._items.keys()

    def items(self) -> ItemsView[str, T]:
        return self._items.items()

    def values(self) -> ValuesView[T]:
        return self._items.values()

    def update_reference(self, old_name: str, new_name: str, resource: T) -> None:
        """Update the collection with a new item."""
        if not isinstance(resource, self._ref_class):
            raise TypeError(f"Expected instance of {self._ref_class.__name__}, got {type(resource).__name__}")
        self._items[new_name] = resource
        if old_name in self._items:
            del self._items[old_name]


class ObjectReferenceProtocol(Protocol[T]):
    @property
    def collection(self) -> ObjectCollection[T]: ...

    @property
    def name(self) -> str: ...

    @property
    def root(self) -> "Root": ...


class ObjectReferenceMixin(Generic[T]):
    @property
    def _connection(self: ObjectReferenceProtocol[T]) -> SnowflakeConnection:
        return self.collection._connection

    @property
    def root(self: ObjectReferenceProtocol[T]) -> "Root":
        """The Root object this reference belongs to."""
        return self.collection.root

    def __repr__(self: ObjectReferenceProtocol[T]) -> str:
        type_name = type(self).__name__
        return f"<{type_name}: {self.name!r}>"



import copy

import pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_tag_template


def test_rename(tags):
    original_name = random_string(10, "test_original_tag_")
    new_name = random_string(10, "test_renamed_tag_")

    tag = copy.deepcopy(test_tag_template)
    tag.name = original_name
    tag_handle = tags.create(tag)

    try:
        fetched_tag = tag_handle.fetch()
        assert fetched_tag.name.upper() == original_name.upper()

        tag_handle.rename(new_name)
        fetched_tag = tag_handle.fetch()

        assert fetched_tag.name == new_name.upper()
        assert fetched_tag.schema_name == tags.schema.name.upper()
        assert fetched_tag.database_name == tags.database.name.upper()

        with pytest.raises(NotFoundError):
            tags[original_name].fetch()

    finally:
        tag_handle.drop(if_exists=True)


def test_rename_nonexistent_tag(tags):
    nonexistent_name = random_string(10, "test_nonexistent_tag_")
    new_name = random_string(10, "test_new_name_tag_")

    with pytest.raises(NotFoundError):
        tags[nonexistent_name].rename(new_name)

    tags[nonexistent_name].rename(new_name, if_exists=True)


def test_rename_tag_cross_schema(tags, temp_schema):
    original_name = random_string(10, "test_cross_schema_tag_")
    new_name = random_string(10, "test_renamed_cross_schema_tag_")

    tag = copy.deepcopy(test_tag_template)
    tag.name = original_name
    tag_handle = tags.create(tag)

    try:
        tag_handle.rename(new_name, target_schema=temp_schema.name)

        fetched_tag = tag_handle.fetch()
        assert fetched_tag.name == new_name.upper()
        assert fetched_tag.schema_name == temp_schema.name.upper()
        assert fetched_tag.database_name == tags.database.name.upper()

        with pytest.raises(NotFoundError):
            tags[original_name].fetch()

    finally:
        tag_handle.drop(if_exists=True)


def test_rename_across_database(tags, temp_db):
    original_name = random_string(10, "test_cross_db_tag_")
    new_name = random_string(10, "test_renamed_cross_db_tag_")

    tag = copy.deepcopy(test_tag_template)
    tag.name = original_name
    tag_handle = tags.create(tag)

    try:
        tag_handle.rename(new_name, target_database=temp_db.name, target_schema="PUBLIC")

        fetched_tag = tag_handle.fetch()
        assert fetched_tag.name == new_name.upper()
        assert fetched_tag.schema_name == "PUBLIC"
        assert fetched_tag.database_name == temp_db.name.upper()

        with pytest.raises(NotFoundError):
            tags[original_name].fetch()

    finally:
        tag_handle.drop(if_exists=True)



# coding: utf-8

# flake8: noqa

"""
Snowflake Notebook API.

The Snowflake Notebook API is a REST API that you can use to access, update, and perform certain actions on Notebook resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.notebook._generated.api.notebook_api import NotebookApi

# import models into sdk package
from snowflake.core.notebook._generated.models.error_response import ErrorResponse
from snowflake.core.notebook._generated.models.notebook import Notebook
from snowflake.core.notebook._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.notebook._generated.models.success_response import SuccessResponse
from snowflake.core.notebook._generated.models.tag_assignment import TagAssignment
from snowflake.core.notebook._generated.models.tag_reference import TagReference
from snowflake.core.notebook._generated.models.version_details import VersionDetails



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import pytest

from snowflake.core.exceptions import APIError


pytestmark = [pytest.mark.skip_gov]


def test_resume_suspend(compute_pools, temp_cp):
    assert compute_pools[temp_cp.name].fetch().state in ("SUSPENDED", "STOPPING")

    compute_pools[temp_cp.name].resume()
    assert compute_pools[temp_cp.name].fetch().state in ("IDLE", "RUNNING", "STARTING", "ACTIVE")

    # it's already resumed
    with pytest.raises(APIError):
        compute_pools[temp_cp.name].resume()

    compute_pools[temp_cp.name].suspend()
    assert compute_pools[temp_cp.name].fetch().state in ("SUSPENDED", "STOPPING")

    # suspend when it is already suspended
    compute_pools[temp_cp.name].suspend()
    assert compute_pools[temp_cp.name].fetch().state in ("SUSPENDED", "STOPPING")



import pytest

from snowflake.core._internal.root_configuration import RootConfiguration
from snowflake.core._root import Root


@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
def test_extracting_tokens(root):
    # Be careful with failures here not to print tokens.
    #  To avoid printing local variables use pytest.fail
    if root._session_token is None:
        pytest.fail("session token should not be None")
    if root._master_token is None:
        pytest.fail("master token should not be None")


@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
def test_init_root_with_user_agents(connection):
    root_config = RootConfiguration()
    root_config.append_user_agent("Snowflake", "1.2.3")
    assert "Snowflake/1.2.3" in Root(connection, root_config).root_config.get_user_agents()

    with pytest.raises(ValueError):
        root_config.append_user_agent("python_api")
    with pytest.raises(ValueError):
        root_config.append_user_agent("python_api", "1.2")

    root_config.append_user_agent("custom_ui")

    new_root = Root(connection, root_config)
    assert "Snowflake/1.2.3" in new_root.root_config.get_user_agents()
    assert "custom_ui" in new_root.root_config.get_user_agents()



import copy

from contextlib import suppress

import pytest as pytest

from snowflake.core.exceptions import APIError, NotFoundError
from snowflake.core.table import TableColumn, UniqueKey
from tests.utils import random_string

from .conftest import assert_table, test_table_template


def test_create_or_alter(tables, database, schema, root):
    table_name = random_string(10, "test_table_")
    table_handle = tables[table_name]
    try:
        test_table = copy.deepcopy(test_table_template)
        test_table.name = table_name
        table_handle.create_or_alter(test_table)  # new table is created for the first time.
        fetched = table_handle.fetch()
        assert_table(fetched, table_name, database, schema, True, rows=0)
        assert fetched.comment == "test table"

        test_table_v2 = copy.deepcopy(test_table_template)
        test_table_v2.name = table_name
        test_table_v2.enable_schema_evolution = False
        test_table_v2.change_tracking = False
        test_table_v2.data_retention_time_in_days = None
        test_table_v2.max_data_extension_time_in_days = None
        test_table_v2.default_ddl_collation = "en"
        test_table_v2.columns[1].nullable = False
        test_table_v2.columns.append(TableColumn(name="c4", datatype="text"))
        test_table_v2.constraints[0].name = "pk2"
        test_table_v2.constraints[1].name = "uk2"
        test_table_v2.comment = "test table 2"
        table_handle.create_or_alter(test_table_v2)
        fetched2 = table_handle.fetch()
        assert fetched2.enable_schema_evolution is False
        assert fetched2.change_tracking is False
        assert len(fetched2.columns) == 4
        assert fetched2.columns[3].name == "C4"
        assert fetched2.columns[3].datatype.upper() in ["TEXT", "VARCHAR(16777216)"]
        assert fetched2.columns[3].nullable is True
        assert fetched2.columns[3].autoincrement_start is None
        assert fetched2.columns[3].autoincrement_increment is None
        if isinstance(fetched2.constraints[0], UniqueKey):
            # Order might swaped
            fetched2.constraints[0], fetched2.constraints[1] = fetched2.constraints[1], fetched2.constraints[0]
        assert fetched2.constraints[0].name == "PK2"
        assert fetched2.constraints[1].name == "UK2"
        assert fetched2.data_retention_time_in_days == 1
        assert fetched2.max_data_extension_time_in_days == 14
        assert fetched2.columns[3].autoincrement is None

        # Make sure that issuing an empty alter doesn't create a malformed SQL
        table_handle.create_or_alter(test_table_v2)
    finally:
        with suppress(NotFoundError):
            table_handle.drop()


def test_create_or_alter_table_negative_no_columns(table_handle):
    fetched_table = table_handle.fetch()
    fetched_table.columns = None
    with pytest.raises(APIError):
        table_handle.create_or_alter(fetched_table)
    # assert error.match("Columns must be provided for create_or_alter")


def test_create_or_alter_table_negative_remove_columns(table_handle):
    deep_fetched = table_handle.fetch()
    deep_fetched.columns.pop(-1)

    table_handle.create_or_alter(deep_fetched)
    deep_fetched = table_handle.fetch()
    assert len(deep_fetched.columns) == 2


def test_create_or_update_deprecated(tables, database, schema, root):
    with pytest.warns(DeprecationWarning, match="method is deprecated; use"):
        table_name = random_string(10, "test_table_")
        table_handle = tables[table_name]
        try:
            test_table = copy.deepcopy(test_table_template)
            test_table.name = table_name
            table_handle.create_or_alter(test_table)  # new table is created for the first time.
            fetched = table_handle.fetch()
            assert_table(fetched, table_name, database, schema, True, rows=0)
            assert fetched.comment == "test table"

            test_table_v2 = copy.deepcopy(test_table_template)
            test_table_v2.name = table_name
            test_table_v2.enable_schema_evolution = False
            test_table_v2.change_tracking = False
            test_table_v2.data_retention_time_in_days = None
            test_table_v2.max_data_extension_time_in_days = None
            test_table_v2.default_ddl_collation = "en"
            test_table_v2.columns[1].nullable = False
            test_table_v2.columns.append(TableColumn(name="c4", datatype="text"))
            test_table_v2.constraints[0].name = "pk2"
            test_table_v2.constraints[1].name = "uk2"
            test_table_v2.comment = "test table 2"
            table_handle.create_or_alter(test_table_v2)
            fetched2 = table_handle.fetch()
            assert fetched2.enable_schema_evolution is False
            assert fetched2.change_tracking is False
            assert len(fetched2.columns) == 4
            assert fetched2.columns[3].name == "C4"
            assert fetched2.columns[3].datatype.upper() in ["TEXT", "VARCHAR(16777216)"]
            assert fetched2.columns[3].nullable is True
            assert fetched2.columns[3].autoincrement_start is None
            assert fetched2.columns[3].autoincrement_increment is None
            if isinstance(fetched2.constraints[0], UniqueKey):
                # Order might swaped
                fetched2.constraints[0], fetched2.constraints[1] = fetched2.constraints[1], fetched2.constraints[0]
            assert fetched2.constraints[0].name == "PK2"
            assert fetched2.constraints[1].name == "UK2"
            assert fetched2.data_retention_time_in_days == 1
            assert fetched2.max_data_extension_time_in_days == 14
            assert fetched2.columns[3].autoincrement is None

            # Make sure that issuing an empty alter doesn't create a malformed SQL
            table_handle.create_or_update(test_table_v2)
        finally:
            with suppress(NotFoundError):
                table_handle.drop()



"""
Snowflake Secret API.

The Snowflake Secret API is a REST API that you can use to access, update, and perform certain actions on Secret resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.secret._generated.models.secret import Secret


class SymmetricKeySecret(Secret):
    """A model object representing the SymmetricKeySecret resource.

    Constructs an object of type SymmetricKeySecret with the provided properties.

    Parameters
    __________

    name : str
        Name of the secret
    algorithm : str
        Key secret's algorithm name

    comment : str, optional
        user comment associated to an object in the dictionary

    created_on : datetime, optional
        Date and time when the secret was created — **Read-only:** *any user-provided value will be ignored.*

    database_name : str, optional
        Database in which the secret is stored — **Read-only:** *any user-provided value will be ignored.*

    schema_name : str, optional
        Schema in which the secret is stored — **Read-only:** *any user-provided value will be ignored.*

    owner : str, optional
        Role that owns the secret — **Read-only:** *any user-provided value will be ignored.*

    owner_role_type : str, optional
        The type of role that owns the secret — **Read-only:** *any user-provided value will be ignored.*
    """

    algorithm: StrictStr

    __properties = ["type", "comment", "name", "created_on", "database_name", "schema_name", "owner", "owner_role_type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SymmetricKeySecret:
        """Create an instance of SymmetricKeySecret from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["type"] = Secret.get_child_model_discriminator_value("SymmetricKeySecret")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> SymmetricKeySecret:
        """Create an instance of SymmetricKeySecret from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return SymmetricKeySecret.model_validate(obj)

        _obj = SymmetricKeySecret.model_validate(
            {
                "comment": obj.get("comment"),
                "name": obj.get("name"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "algorithm": obj.get("algorithm"),
            }
        )

        return _obj


class SymmetricKeySecretModel(Secret):
    def __init__(
        self,
        name: str,
        algorithm: str,
        # optional properties
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
    ):
        """A model object representing the SymmetricKeySecret resource.

        Constructs an object of type SymmetricKeySecret with the provided properties.

        Parameters
        __________

        name : str
            Name of the secret
        algorithm : str
            Key secret's algorithm name

        comment : str, optional
            user comment associated to an object in the dictionary

        created_on : datetime, optional
            Date and time when the secret was created.

        database_name : str, optional
            Database in which the secret is stored

        schema_name : str, optional
            Schema in which the secret is stored

        owner : str, optional
            Role that owns the secret

        owner_role_type : str, optional
            The type of role that owns the secret
        """
        super().__init__(
            comment=comment,
            name=name,
            created_on=created_on,
            database_name=database_name,
            schema_name=schema_name,
            owner=owner,
            owner_role_type=owner_role_type,
        )
        self.algorithm = algorithm

    __properties = ["type", "comment", "name", "created_on", "database_name", "schema_name", "owner", "owner_role_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return SymmetricKeySecret(
            comment=self.comment,
            name=self.name,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            algorithm=self.algorithm,
        )

    @classmethod
    def _from_model(cls, model) -> SymmetricKeySecretModel:
        return SymmetricKeySecretModel(
            comment=model.comment,
            name=model.name,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            algorithm=model.algorithm,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a SymmetricKeySecret.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the SymmetricKeySecret object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> SymmetricKeySecretModel:
        """Create an instance of SymmetricKeySecret from a dict.

        This method constructs a SymmetricKeySecret object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        SymmetricKeySecret
            A SymmetricKeySecret object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(SymmetricKeySecret.from_dict(obj))


SymmetricKeySecret._model_class = SymmetricKeySecretModel



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import time
import typing

import pytest

from snowflake.core.task import StoredProcedureCall, Task
from snowflake.core.task.context import TaskContext

from ..utils import get_task_history, random_object_name


if typing.TYPE_CHECKING:
    pass


task_name1 = random_object_name()
task_name2 = random_object_name()


PACKAGES = [
    # Clients do not need to use `atpublic`; it's a convenient decorator used in TaskContext so it needs to be
    # named in the stored procedure packages list.
    "atpublic",
    "snowflake-snowpark-python",
    "snowflake.core",
]


@pytest.mark.usefixtures("anaconda_package_available")
def test_create_task_from_python_function(tasks, session):
    from snowflake.snowpark import Session

    def f(session: Session, table_name: str) -> str:
        context = TaskContext(session)
        value = context.get_current_task_name()
        session.sql(f"insert into {table_name} values(1, '{value}')").collect()
        context.set_return_value(value)
        return session.sql("select 'snowflake'").collect()[0][0]

    def g(session: Session, table_name: str, predecessor_name: str) -> str:
        def assert_none(value: typing.Any):
            if value is not None:
                raise ValueError("assert_none error.")

        def assert_not_none(value: typing.Any):
            if value is None:
                raise ValueError("assert_not_none error.")

        context = TaskContext(session)
        value1 = context.get_predecessor_return_value()
        value2 = context.get_predecessor_return_value(predecessor_name)

        # When using python's own assert statement, _pytest was used but it's not available.
        # Don't know why this happens but the following works.
        assert_not_none(context.get_task_graph_config())
        assert_not_none(context.get_task_graph_config_property("a"))
        assert_not_none(context.get_current_task_name())
        assert_not_none(context.get_current_root_task_name())
        assert_not_none(context.get_current_task_short_name())
        assert_not_none(context.get_current_root_task_uuid())
        assert_not_none(context.get_current_task_graph_original_schedule())
        assert_not_none(context.get_current_task_graph_run_group_id())
        assert_none(context.get_last_successful_task_graph_run_group_id())
        assert_none(context.get_last_successful_task_graph_original_schedule())

        session.sql(f"insert into {table_name} values(2, '{value1}.{value2}')").collect()
        return ""

    stage_name = random_object_name()
    table_name = random_object_name()
    warehouse = session.get_current_warehouse()
    try:
        session.sql(f"create or replace stage {stage_name}").collect()
        session.sql(f"create or replace table {table_name} (id int, str varchar)").collect()

        # create t1
        t1 = Task(
            task_name1,
            StoredProcedureCall(f, args=[table_name], stage_location=stage_name, packages=PACKAGES),
            warehouse=warehouse,
            config={"a": 1},
        )
        with pytest.raises(ValueError) as ex_info:
            _ = t1.sql_definition
        assert "definition of this task can only be retrieved after creating the task" in str(ex_info)
        task1 = tasks.create(t1)
        assert "LANGUAGE PYTHON" in t1.sql_definition

        # create t2
        t2 = Task(
            task_name2,
            StoredProcedureCall(g, args=[table_name, task_name1.upper()], stage_location=stage_name, packages=PACKAGES),
            predecessors=[task1.name],
            warehouse=None,
        )
        with pytest.raises(ValueError) as ex_info:
            _ = t2.sql_definition
        assert "definition of this task can only be retrieved after creating the task" in str(ex_info)
        task2 = tasks.create(t2)
        assert "LANGUAGE PYTHON" in t2.sql_definition

        # resume and execute
        task2.resume()
        task1.execute()

        # get the history
        time.sleep(1)
        time_count = 0
        time_limit = 120
        task_name = task_name1
        while True:
            result = get_task_history(session, task_name)
            assert len(result) > 0
            state = result[0]["STATE"]
            if state in ["SCHEDULED", "EXECUTING"]:
                time.sleep(1)
                time_count += 1
                if time_count > time_limit:  # run for 2 min
                    pytest.xfail("Flaky test, fix trace by SNOW-1317265")
                    raise ValueError(f"Running more than {time_limit} seconds. task: {task_name}, result: {result}")
                continue
            elif state == "SUCCEEDED":
                assert result[0]["ERROR_CODE"] is None
                if task_name == task_name2:
                    break
                task_name = task_name2
            else:
                raise ValueError(f"unexpected state: {state}. {result}")

        result = session.table(table_name).sort("id").collect()
        assert len(result) == 2
        assert result[0][1].count(task_name1.upper()) == 1
        assert result[1][1].count(task_name1.upper()) == 2
    finally:
        session.sql(f"drop stage if exists {stage_name}").collect()
        session.sql(f"drop table if exists {table_name}").collect()
        session.sql(f"drop task if exists {task_name1}").collect()
        session.sql(f"drop task if exists {task_name2}").collect()


@pytest.mark.usefixtures("anaconda_package_available")
def test_create_task_from_python_stored_proc(tasks, session):
    from snowflake.snowpark import Session

    def f(session: Session, table_name: str) -> int:
        session.sql(f"insert into {table_name} values(1, 'abc')").collect()
        return 0

    def g(session: Session, table_name: str) -> float:
        session.sql(f"insert into {table_name} values(2, 'def')").collect()
        return 1.0

    stage_name = random_object_name()
    table_name = random_object_name()
    sp1_name = random_object_name()
    sp2_name = random_object_name()
    warehouse = session.get_current_warehouse()
    try:
        session.sql(f"create or replace stage {stage_name}").collect()
        session.sql(f"create or replace table {table_name} (id int, str varchar)").collect()

        sproc1 = session.sproc.register(
            f, stage_location=stage_name, packages=PACKAGES, name=sp1_name, replace=True, is_permanent=True
        )
        sproc2 = session.sproc.register(
            g, stage_location=stage_name, packages=PACKAGES, name=sp2_name, replace=True, is_permanent=True
        )
        # create t1
        t1 = Task(task_name1, StoredProcedureCall(sproc1, args=[table_name]), warehouse=None)
        with pytest.raises(ValueError) as ex_info:
            _ = t1.sql_definition
        assert "definition of this task can only be retrieved after creating the task" in str(ex_info)
        task1 = tasks.create(t1)
        assert "CALL" in t1.sql_definition

        # create t2
        t2 = Task(
            task_name2, StoredProcedureCall(sproc2, args=[table_name]), predecessors=[task1.name], warehouse=warehouse
        )
        with pytest.raises(ValueError) as ex_info:
            _ = t2.sql_definition
        assert "definition of this task can only be retrieved after creating the task" in str(ex_info)
        task2 = tasks.create(t2)
        assert "CALL" in t2.sql_definition

        # resume and execute
        task2.resume()
        task1.execute()

        # get the history
        time.sleep(1)
        time_count = 0
        time_limit = 120
        task_name = task_name1
        while True:
            result = get_task_history(session, task_name)
            assert len(result) > 0
            state = result[0]["STATE"]
            if state in ["SCHEDULED", "EXECUTING"]:
                time.sleep(1)
                time_count += 1
                if time_count > time_limit:  # run for 2 min
                    pytest.xfail("Flaky test, fix trace by SNOW-1317265")
                    raise ValueError(f"Running more than {time_limit} seconds. task: {task_name}, result: {result}")
                continue
            elif state == "SUCCEEDED":
                assert result[0]["ERROR_CODE"] is None
                if task_name == task_name2:
                    break
                task_name = task_name2
            else:
                raise ValueError(f"unexpected state: {state}")

        result = session.table(table_name).sort("id").collect()
        assert len(result) == 2
        assert result[0][1] == "abc"
        assert result[1][1] == "def"
    finally:
        session.sql(f"drop stage if exists {stage_name}").collect()
        session.sql(f"drop table if exists {table_name}").collect()
        session.sql(f"drop task if exists {task_name1}").collect()
        session.sql(f"drop task if exists {task_name2}").collect()
        session.sql(f"drop procedure if exists {sp1_name}(string)").collect()
        session.sql(f"drop procedure if exists {sp2_name}(string)").collect()


@pytest.mark.snowpark
def test_create_task_from_python_function_requires_stage_location(tasks):
    with pytest.raises(ValueError, match="stage_location has to be specified when func is a Python function"):
        StoredProcedureCall(lambda: 1)


@pytest.mark.snowpark
def test_task_context(tasks, session):
    from snowflake.snowpark.exceptions import SnowparkSQLException

    # we are not able to test it in a real task, so just call functions here
    task_context = TaskContext(session)

    with pytest.raises(SnowparkSQLException, match="must be called from within a task"):
        task_context.set_return_value("1")

    with pytest.raises(SnowparkSQLException, match="must be called from within a task"):
        task_context.get_predecessor_return_value()

    with pytest.raises(ValueError):
        task_context.get_predecessor_return_value("invalid name")

    # this function can actually be called outside of a task...
    assert not task_context.get_current_task_name()



# coding: utf-8

# flake8: noqa
"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.stage._generated.models.aws_credentials import AwsCredentials
from snowflake.core.stage._generated.models.azure_credentials import AzureCredentials
from snowflake.core.stage._generated.models.credentials import Credentials
from snowflake.core.stage._generated.models.error_response import ErrorResponse
from snowflake.core.stage._generated.models.file_transfer_material import FileTransferMaterial
from snowflake.core.stage._generated.models.presigned_url_request import PresignedUrlRequest
from snowflake.core.stage._generated.models.stage import Stage
from snowflake.core.stage._generated.models.stage_directory_table import StageDirectoryTable
from snowflake.core.stage._generated.models.stage_encryption import StageEncryption
from snowflake.core.stage._generated.models.stage_file import StageFile
from snowflake.core.stage._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.stage._generated.models.success_response import SuccessResponse

__all__ = [
    "AwsCredentials",
    "AzureCredentials",
    "Credentials",
    "ErrorResponse",
    "FileTransferMaterial",
    "PresignedUrlRequest",
    "Stage",
    "StageDirectoryTable",
    "StageEncryption",
    "StageFile",
    "SuccessAcceptedResponse",
    "SuccessResponse",
]



"""
Cortex Chat API.

OpenAPI 3.0 specification for the Cortex Chat API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr


class ChatRequestSearchServicesInner(BaseModel):
    """A model object representing the ChatRequestSearchServicesInner resource.

    Constructs an object of type ChatRequestSearchServicesInner with the provided properties.

    Parameters
    __________
    name : str
        The name of the search service.
    max_results : int, optional
        The number of search results used in LLM prompt.
    title_column : str, optional
        The title of the document.
    id_column : str, optional
        The ID of the document.
    filter : object, optional
        Filter query.
    experimental : object, optional
        reserved
    """

    name: StrictStr

    max_results: Optional[StrictInt] = None

    title_column: Optional[StrictStr] = None

    id_column: Optional[StrictStr] = None

    filter: Optional[Dict[str, Any]] = None

    experimental: Optional[Dict[str, Any]] = None

    __properties = ["name", "max_results", "title_column", "id_column", "filter", "experimental"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ChatRequestSearchServicesInner:
        """Create an instance of ChatRequestSearchServicesInner from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ChatRequestSearchServicesInner:
        """Create an instance of ChatRequestSearchServicesInner from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ChatRequestSearchServicesInner.model_validate(obj)

        _obj = ChatRequestSearchServicesInner.model_validate(
            {
                "name": obj.get("name"),
                "max_results": obj.get("max_results"),
                "title_column": obj.get("title_column"),
                "id_column": obj.get("id_column"),
                "filter": obj.get("filter"),
                "experimental": obj.get("experimental"),
            }
        )

        return _obj


class ChatRequestSearchServicesInnerModel:
    def __init__(
        self,
        name: str,
        # optional properties
        max_results: Optional[int] = None,
        title_column: Optional[str] = None,
        id_column: Optional[str] = None,
        filter: Optional[object] = None,
        experimental: Optional[object] = None,
    ):
        """A model object representing the ChatRequestSearchServicesInner resource.

        Constructs an object of type ChatRequestSearchServicesInner with the provided properties.

        Parameters
        __________
        name : str
            The name of the search service.
        max_results : int, optional
            The number of search results used in LLM prompt.
        title_column : str, optional
            The title of the document.
        id_column : str, optional
            The ID of the document.
        filter : object, optional
            Filter query.
        experimental : object, optional
            reserved
        """
        self.name = name
        self.max_results = max_results
        self.title_column = title_column
        self.id_column = id_column
        self.filter = filter
        self.experimental = experimental

    __properties = ["name", "max_results", "title_column", "id_column", "filter", "experimental"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ChatRequestSearchServicesInner(
            name=self.name,
            max_results=self.max_results,
            title_column=self.title_column,
            id_column=self.id_column,
            filter=self.filter,
            experimental=self.experimental,
        )

    @classmethod
    def _from_model(cls, model) -> ChatRequestSearchServicesInnerModel:
        return ChatRequestSearchServicesInnerModel(
            name=model.name,
            max_results=model.max_results,
            title_column=model.title_column,
            id_column=model.id_column,
            filter=model.filter,
            experimental=model.experimental,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ChatRequestSearchServicesInner.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ChatRequestSearchServicesInner object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ChatRequestSearchServicesInnerModel:
        """Create an instance of ChatRequestSearchServicesInner from a dict.

        This method constructs a ChatRequestSearchServicesInner object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ChatRequestSearchServicesInner
            A ChatRequestSearchServicesInner object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ChatRequestSearchServicesInner.from_dict(obj))


ChatRequestSearchServicesInner._model_class = ChatRequestSearchServicesInnerModel



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.notification_integration._generated.api.notification_integration_api import (
    NotificationIntegrationApi,
)

__all__ = [
    "NotificationIntegrationApi",
]



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.password_policy import PasswordPolicy, PasswordPolicyResource
from snowflake.core.password_policy._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


@pytest.fixture
def password_policies(schema):
    return schema.password_policies


@pytest.fixture
def password_policy(password_policies):
    return password_policies["my_password_policy"]


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
PASSWORD_POLICY = PasswordPolicy(
    name="my_password_policy",
    password_min_length=8,
    password_max_length=256,
    password_min_upper_case_chars=1,
    password_min_lower_case_chars=1,
    password_min_numeric_chars=1,
    password_min_special_chars=1,
    password_min_age_days=1,
    password_max_age_days=90,
    password_max_retries=5,
    password_lockout_time_mins=15,
    password_history=24,
    comment="Test password policy",
)


def test_create_password_policy(fake_root, password_policies):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/password-policies")
    kwargs = extra_params(
        query_params=[],
        body={
            "name": "my_password_policy",
            "password_min_length": 8,
            "password_max_length": 256,
            "password_min_upper_case_chars": 1,
            "password_min_lower_case_chars": 1,
            "password_min_numeric_chars": 1,
            "password_min_special_chars": 1,
            "password_min_age_days": 1,
            "password_max_age_days": 90,
            "password_max_retries": 5,
            "password_lockout_time_mins": 15,
            "password_history": 24,
            "comment": "Test password policy",
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        pp_res = password_policies.create(PASSWORD_POLICY)
        assert isinstance(pp_res, PasswordPolicyResource)
        assert pp_res.name == "my_password_policy"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = password_policies.create_async(PASSWORD_POLICY)
        assert isinstance(op, PollingOperation)
        pp_res = op.result()
        assert pp_res.name == "my_password_policy"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_password_policy(fake_root, password_policies):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/password-policies")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        password_policies.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = password_policies.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit_deprecation_warning(fake_root, password_policies):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/password-policies?showLimit=10",
    )
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            list(password_policies.iter(show_limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            password_policies.iter_async(show_limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_limit_and_show_limit_conflict(password_policies):
    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        list(password_policies.iter(limit=10, show_limit=5))

    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        password_policies.iter_async(limit=10, show_limit=5).result()


def test_fetch_password_policy(fake_root, password_policy):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(PASSWORD_POLICY.to_json())
        password_policy.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(PASSWORD_POLICY.to_json())
        op = password_policy.fetch_async()
        assert isinstance(op, PollingOperation)
        policy = op.result()
        assert policy.to_dict() == PASSWORD_POLICY.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_password_policy(fake_root, password_policy):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        password_policy.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = password_policy.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_rename_password_policy(fake_root, password_policy, password_policies):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy:rename?targetName=new_password_policy",
    )
    kwargs = extra_params(query_params=[("targetName", "new_password_policy")])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        password_policy.rename("new_password_policy")
        assert password_policy.name == "new_password_policy"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        pp_res = password_policies["another_password_policy"]
        op = pp_res.rename_async("new_password_policy")
        assert isinstance(op, PollingOperation)
        op.result()
        assert pp_res.name == "new_password_policy"
    args2 = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/password-policies/another_password_policy:rename?targetName=new_password_policy",
    )
    mocked_request.assert_called_once_with(*args2, **kwargs)


def test_rename_password_policy_with_options(fake_root, password_policy):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy:rename?ifExists=True&targetDatabase=other_db&targetSchema=other_schema&targetName=new_password_policy",
    )
    kwargs = extra_params(
        query_params=[
            ("ifExists", True),
            ("targetDatabase", "other_db"),
            ("targetSchema", "other_schema"),
            ("targetName", "new_password_policy"),
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        password_policy.rename(
            "new_password_policy", target_database="other_db", target_schema="other_schema", if_exists=True
        )
        assert password_policy.name == "new_password_policy"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, password_policy, tag):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy:set-tags",
    )
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        password_policy.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = password_policy.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, password_policy, tag):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy:unset-tags",
    )
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        password_policy.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = password_policy.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, password_policy):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/password-policies/my_password_policy:get-tags",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert password_policy.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = password_policy.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.user_defined_function._generated.api.user_defined_function_api import UserDefinedFunctionApi

__all__ = [
    "UserDefinedFunctionApi",
]



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
import os
import time
import typing
import uuid

from contextlib import contextmanager, suppress
from typing import Any, Iterator

import pytest

from pydantic import StrictStr

import snowflake.connector

from snowflake.connector import SnowflakeConnection
from snowflake.core import CreateMode, Root
from snowflake.core.alert import AlertCollection
from snowflake.core.api_integration import ApiIntegrationCollection
from snowflake.core.catalog_integration import CatalogIntegration
from snowflake.core.compute_pool import ComputePoolCollection
from snowflake.core.cortex.chat_service import CortexChatService
from snowflake.core.cortex.embed_service import CortexEmbedService
from snowflake.core.cortex.lite_agent_service import CortexAgentService
from snowflake.core.cortex.search_service import CortexSearchServiceCollection
from snowflake.core.database import DatabaseCollection, DatabaseResource
from snowflake.core.database_role import DatabaseRoleCollection
from snowflake.core.event_table import EventTableCollection
from snowflake.core.external_volume import ExternalVolumeCollection
from snowflake.core.function import FunctionCollection
from snowflake.core.grant._grants import Grants
from snowflake.core.iceberg_table import IcebergTableCollection
from snowflake.core.image_repository import ImageRepositoryCollection
from snowflake.core.network_policy import NetworkPolicyCollection
from snowflake.core.network_rule import NetworkRuleCollection
from snowflake.core.notebook import NotebookCollection
from snowflake.core.notification_integration import NotificationIntegrationCollection
from snowflake.core.password_policy import PasswordPolicyCollection
from snowflake.core.pipe import PipeCollection
from snowflake.core.procedure import ProcedureCollection
from snowflake.core.role import RoleCollection
from snowflake.core.schema import SchemaCollection, SchemaResource
from snowflake.core.secret import SecretCollection
from snowflake.core.sequence import SequenceCollection
from snowflake.core.service import ServiceCollection
from snowflake.core.stage import StageCollection
from snowflake.core.stream import StreamCollection
from snowflake.core.streamlit import StreamlitCollection
from snowflake.core.table import TableCollection
from snowflake.core.tag import Tag, TagCollection, TagResource
from snowflake.core.user import UserCollection
from snowflake.core.user_defined_function import UserDefinedFunctionCollection
from snowflake.core.view import ViewCollection
from snowflake.core.warehouse import WarehouseCollection, WarehouseResource
from tests.integ.utils import backup_role, random_string

from ..utils import ensure_snowflake_version, is_prod_or_preprod
from .fixtures.backup_objects import (  # noqa: F401 # pylint: disable=unused-import
    backup_database_schema,
    backup_warehouse_fixture,
)
from .fixtures.constants import (
    DEFAULT_IR_URL,
    TEST_DATABASE,
    TEST_IMAGE_REPO,
    TEST_IMAGE_REPO_QUALIFIED_SCHEMA,
    TEST_SCHEMA,
    TEST_SHARED_TAG_DATABASE,
    TEST_SHARED_TAG_NAME,
    TEST_SHARED_TAG_SCHEMA,
    TEST_SHARED_TAG_VALUE,
    TEST_WAREHOUSE,
)
from .fixtures.objects_setup import (  # noqa: F401 # pylint: disable=unused-import
    setup_basic,
    setup_credentials_fixture,
    shared_tag_setup,
    spcs_setup_objects,
    warehouse_setup,
)
from .fixtures.pre_checks import (  # noqa: F401 # pylint: disable=unused-import
    anaconda_package_available,
    maven_snowpark_jar_available,
    qa_mode_enabled,
    shared_database_available,
    skip_for_snowflake_account,
)
from .fixtures.temp_objects import (  # noqa: F401 # pylint: disable=unused-import
    temp_cp,
    temp_customer_account_cursor,
    temp_customer_account_root,
    temp_customer_account_with_org_admin,
    temp_customer_organization,
    temp_db,
    temp_db_case_sensitive,
    temp_directory_table,
    temp_ir,
    temp_schema,
    temp_schema_case_sensitive,
    temp_schema_upper_case,
    temp_service,
    temp_service_from_spec_inline,
    temp_stage,
    temp_stage_case_sensitive,
    temp_table,
    temp_tag_upper_case,
    test_schema,
)
from .utils import connection_config, connection_keys


if typing.TYPE_CHECKING:
    from snowflake.snowpark import Session

RUNNING_IN_NOTEBOOK = "RUNNING_IN_NOTEBOOK" in os.environ
RUNNING_IN_STOREDPROC = "RUNNING_IN_STOREDPROC" in os.environ

SF_ACCOUNT_CONFIG_KEY = "__sf_account__"
SF_ACCOUNT = "snowflake"


@pytest.fixture(scope="session")
def shared_compute_pool(spcs_setup_objects, connection):  # noqa: F811
    yield spcs_setup_objects.compute_pool
    # clean up services and jobs in the compute pool
    print(f"Cleaning up services and jobs in the compute pool {spcs_setup_objects.compute_pool}")
    with connection.cursor() as cur:
        cur.execute(f"alter compute pool {spcs_setup_objects.compute_pool} stop all")


@pytest.fixture(scope="session")
def shared_account_tag(root, shared_tag_setup) -> TagResource:  # noqa: F811
    tags = root.databases[TEST_SHARED_TAG_DATABASE].schemas[TEST_SHARED_TAG_SCHEMA].tags
    tag = tags.create(Tag(name=TEST_SHARED_TAG_NAME), mode=CreateMode.if_not_exists)
    root.connection.execute_string(
        f"ALTER ACCOUNT "
        f"SET TAG {TEST_SHARED_TAG_DATABASE}.{TEST_SHARED_TAG_SCHEMA}.{TEST_SHARED_TAG_NAME}"
        f" = '{TEST_SHARED_TAG_VALUE}'"
    )
    return tags[tag.name]


@pytest.fixture(scope="session")
def instance_family(spcs_setup_objects):  # noqa: F811
    yield spcs_setup_objects.instance_family


def print_help() -> None:
    print(
        """Connection parameter must be specified in parameters.py,
    for example:
CONNECTION_PARAMETERS = {
    'account': 'testaccount',
    'user': 'user1',
    'password': 'test',
    'database': 'testdb',
    'schema': 'public',
}
"""
    )


def pytest_runtest_setup(item):
    # Skip any test marked for Notebook when running on Notebook Environment
    if RUNNING_IN_NOTEBOOK:
        notebook_marker = list(item.iter_markers(name="skip_notebook"))
        if notebook_marker:
            pytest.skip("this test is not supposed to run on Notebook Environment")
    # Skip any test marked for Storedproc when running on Storedproc Environment
    if RUNNING_IN_STOREDPROC:
        storedproc_marker = list(item.iter_markers(name="skip_storedproc"))
        if storedproc_marker:
            pytest.skip("this test is not supposed to run on Storedproc Environment")


@pytest.fixture(scope="session")
def cursor(connection):
    with suppress(Exception):
        yield connection.cursor()


@pytest.fixture(autouse=True)
def min_sf_ver(request, snowflake_version):
    if "min_sf_ver" in request.keywords and len(request.keywords["min_sf_ver"].args) > 0:
        requested_version = request.keywords["min_sf_ver"].args[0]

        ensure_snowflake_version(snowflake_version, requested_version)


@pytest.fixture(scope="session")
def snowflake_version(connection) -> str:
    with connection.cursor() as cur:
        return cur.execute("select current_version()").fetchone()[0].strip()


@pytest.fixture(scope="session")
def snowflake_region(connection) -> str:
    with connection.cursor() as cur:
        return cur.execute("select current_region()").fetchone()[0].strip()


@pytest.fixture(scope="function", autouse=True)
def print_snowflake_version(snowflake_version):
    print(f"Running against Snowflake version: {snowflake_version}")


@pytest.fixture(scope="session")
def db_parameters() -> dict[str, str]:
    if RUNNING_IN_NOTEBOOK or RUNNING_IN_STOREDPROC:
        from snowflake.snowpark.context import get_active_session

        config = {}
        session = get_active_session()
        config["account"] = session.get_current_account()
        config["schema"] = "TESTSCHEMA"
        config["database"] = "TESTDB"
        config["warehouse"] = session.get_current_warehouse()
        #  for notebook run make sure the account you are running on is already set up for SPCS
        config["should_disable_setup_for_spcs"] = "true"
        config["should_disable_setup_for_credentials"] = "true"
        return config

    config = connection_config(override_schema=TEST_SCHEMA, override_database=TEST_DATABASE)
    config["schema"] = "TESTSCHEMA"
    config["database"] = "TESTDB"
    return config


# 2023-06-21(warsaw): WARNING!  If any of these fixtures fail, they will print
# db_parameters in the traceback, and that **will** leak the password.  pytest
# doesn't seem to have any way to suppress the password, and I've tried lots
# of things to get that to work, to no avail.


@pytest.fixture(scope="session")
def session_notebook() -> "Session":
    from snowflake.snowpark.context import get_active_session

    return get_active_session()


@pytest.fixture(scope="session")
def session_default(connection_default) -> "Session":
    from snowflake.snowpark import Session

    return Session.builder.config("connection", connection_default).create()


@pytest.fixture(scope="session")
def session(request) -> "Session":
    if RUNNING_IN_NOTEBOOK or RUNNING_IN_STOREDPROC:
        return request.getfixturevalue("session_notebook")
    else:
        return request.getfixturevalue("session_default")


@pytest.fixture(scope="session")
def connection_notebook(session_notebook) -> SnowflakeConnection:
    return session_notebook.connection


@pytest.fixture(scope="session")
def connection_default(db_parameters) -> SnowflakeConnection:
    _keys = connection_keys()
    with snowflake.connector.connect(
        # This works around SNOW-998521, by forcing JSON results
        **{k: db_parameters[k] for k in _keys if k in db_parameters}
    ) as con:
        yield con


@pytest.fixture(scope="session")
def connection(request) -> SnowflakeConnection:
    if RUNNING_IN_NOTEBOOK or RUNNING_IN_STOREDPROC:
        return request.getfixturevalue("connection_notebook")
    else:
        return request.getfixturevalue("connection_default")


@pytest.fixture(scope="session")
def test_account(db_parameters) -> str:
    return db_parameters["account"].lower().strip()


@pytest.fixture(scope="session")
def root(request, connection) -> Root:
    if RUNNING_IN_NOTEBOOK or RUNNING_IN_STOREDPROC:
        notebook_session = request.getfixturevalue("session_notebook")
        return Root(notebook_session)
    return Root(connection)


@pytest.fixture(scope="session")
def api_integrations(root) -> ApiIntegrationCollection:
    return root.api_integrations


@pytest.fixture(scope="session")
def database(root) -> DatabaseResource:
    return root.databases[TEST_DATABASE]


@pytest.fixture(scope="session")
def schema(schemas) -> SchemaResource:
    return schemas[TEST_SCHEMA]


@pytest.fixture(scope="session")
def warehouse(warehouses, warehouse_setup) -> WarehouseResource:  # noqa: F811
    return warehouses[TEST_WAREHOUSE]


@pytest.fixture(scope="module")
def image_repositories(schema) -> ImageRepositoryCollection:
    return schema.image_repositories


@pytest.fixture(scope="module")
def compute_pools(root) -> ComputePoolCollection:
    return root.compute_pools


@pytest.fixture(scope="session")
def external_volumes(root) -> ExternalVolumeCollection:
    return root.external_volumes


@pytest.fixture(scope="session")
def warehouses(root) -> WarehouseCollection:
    return root.warehouses


@pytest.fixture(scope="session")
def services(schema) -> ServiceCollection:
    return schema.services


@pytest.fixture(scope="session")
def functions(schema) -> FunctionCollection:
    return schema.functions


@pytest.fixture(scope="session")
def notebooks(schema) -> NotebookCollection:
    return schema.notebooks


@pytest.fixture(scope="session")
def streamlits(schema) -> StreamlitCollection:
    return schema.streamlits


@pytest.fixture(scope="session")
def procedures(schema) -> ProcedureCollection:
    return schema.procedures


@pytest.fixture(scope="session")
def database_roles(database) -> DatabaseRoleCollection:
    return database.database_roles


@pytest.fixture(scope="session")
def databases(root) -> DatabaseCollection:
    return root.databases


@pytest.fixture(scope="session")
def notification_integrations(root) -> NotificationIntegrationCollection:
    return root.notification_integrations


@pytest.fixture(scope="session")
def schemas(database) -> SchemaCollection:
    return database.schemas


@pytest.fixture(scope="module")
def roles(root) -> RoleCollection:
    return root.roles


@pytest.fixture(scope="module")
def users(root) -> UserCollection:
    return root.users


@pytest.fixture(scope="module")
def grants(root) -> Grants:
    return root.grants


@pytest.fixture(scope="module")
def network_policies(root) -> NetworkPolicyCollection:
    return root.network_policies


@pytest.fixture(scope="module")
def stages(schema) -> StageCollection:
    return schema.stages


@pytest.fixture(scope="module")
def streams(schema) -> StreamCollection:
    return schema.streams


@pytest.fixture(scope="module")
def pipes(schema) -> PipeCollection:
    return schema.pipes


@pytest.fixture(scope="module")
def tables(schema) -> TableCollection:
    return schema.tables


@pytest.fixture(scope="module")
def iceberg_tables(schema) -> IcebergTableCollection:
    return schema.iceberg_tables


@pytest.fixture(scope="module")
def views(schema) -> ViewCollection:
    return schema.views


@pytest.fixture(scope="session")
def cortex_search_services(schema) -> CortexSearchServiceCollection:
    return schema.cortex_search_services


@pytest.fixture(scope="session")
def cortex_chat_services(root) -> CortexChatService:
    return root.cortex_chat_services


@pytest.fixture(scope="session")
def cortex_embed_service(root) -> CortexEmbedService:
    return root.cortex_embed_service


@pytest.fixture(scope="session")
def cortex_agent_services(root) -> CortexAgentService:
    return root.cortex_agent_services


@pytest.fixture(scope="session")
def event_tables(schema) -> EventTableCollection:
    return schema.event_tables


@pytest.fixture(scope="session")
def alerts(schema) -> AlertCollection:
    return schema.alerts


@pytest.fixture(scope="session")
def catalog_integrations(root) -> CatalogIntegration:
    return root.catalog_integrations


@pytest.fixture(scope="session")
def user_defined_functions(schema) -> UserDefinedFunctionCollection:
    return schema.user_defined_functions


@pytest.fixture(scope="session")
def network_rules(schema) -> NetworkRuleCollection:
    return schema.network_rules


@pytest.fixture(scope="session")
def password_policies(schema) -> PasswordPolicyCollection:
    return schema.password_policies


@pytest.fixture(scope="session")
def secrets(schema) -> SecretCollection:
    return schema.secrets


@pytest.fixture(scope="session")
def sequences(schema) -> SequenceCollection:
    return schema.sequences


@pytest.fixture(scope="session")
def tags(schema) -> TagCollection:
    return schema.tags


@pytest.fixture(scope="module")
def tag(tags) -> Iterator[TagResource]:
    tag = tags.create(Tag(name="TEST_TAG"))
    try:
        yield tags[tag.name]
    finally:
        tag.drop(if_exists=True)


@pytest.fixture(scope="function")
def temp_pypi_api_integration(cursor):
    """Create a temporary PyPI API integration via SQL for artifact repository tests."""
    ai_name = random_string(10, "test_artifact_repo_pypi_integration_")

    # Create PyPI API integration using direct SQL - this is the key for artifact repositories!
    create_sql = f"""
    CREATE API INTEGRATION {ai_name}
    API_PROVIDER = pypi
    ENABLED = true
    COMMENT = 'PyPI API integration for artifact repository integration tests'
    """

    cursor.execute(create_sql)
    try:
        yield ai_name.upper()  # Return uppercase name for consistency
    finally:
        cursor.execute(f"DROP API INTEGRATION IF EXISTS {ai_name}")


@pytest.fixture(scope="module")
def artifact_repositories(schema):
    """Artifact repository collection fixture for integration tests."""
    return schema.artifact_repositories


@pytest.fixture(scope="session")
def imagerepo(connection, spcs_setup_objects) -> str:  # noqa: F811
    # When adding an inlined image repository YAML file, don't hard code the path to the test image
    # repository.  Instead, use this fixture and f-string this value in for the `{imagrepo}` substitution.
    # This way, there's only one thing to change across the entire test suite.
    # Legacy: return 'sfengineering-ss-lprpr-test2.registry
    #    .snowflakecomputing.com/testdb_python/public/ci_image_repository'
    with connection.cursor() as cursor:
        try:
            image_repos = cursor.execute(
                f"SHOW IMAGE REPOSITORIES IN SCHEMA {TEST_IMAGE_REPO_QUALIFIED_SCHEMA};"
            ).fetchall()
            assert TEST_IMAGE_REPO in [x[1].lower() for x in image_repos]
            return DEFAULT_IR_URL
        except Exception:
            cursor.execute(
                f"CREATE IMAGE REPOSITORY IF NOT EXISTS {TEST_IMAGE_REPO_QUALIFIED_SCHEMA}.{TEST_IMAGE_REPO};"
            )
            return cursor.execute(
                f"SHOW IMAGE REPOSITORIES LIKE '{TEST_IMAGE_REPO} IN SCHEMA {TEST_IMAGE_REPO_QUALIFIED_SCHEMA}';"
            ).fetchone()[4]


@pytest.fixture(scope="session")
def external_session_id() -> str:
    return str(uuid.uuid4())


@pytest.fixture(scope="session")
def pat(connection, db_parameters) -> str:
    pat_token_name = "SNOW_API_TEST_TOKEN_" + str(int(time.time() * 1000))
    role = db_parameters.get("role", "SYSADMIN")
    try:
        cursor = connection.cursor()
        token_name, pat_token = cursor.execute(
            f"ALTER USER ADD PROGRAMMATIC ACCESS TOKEN {pat_token_name} "
            f"ROLE_RESTRICTION = {role} DAYS_TO_EXPIRY=1 COMMENT='PAT for SnowAPI tests'"
        ).fetchone()
        assert token_name == pat_token_name
        yield pat_token
    finally:
        connection.cursor().execute(f"ALTER USER REMOVE PROGRAMMATIC ACCESS TOKEN {pat_token_name}")


@pytest.fixture(scope="module")
def pat_with_external_session_connection(request, db_parameters, pat, external_session_id) -> SnowflakeConnection:
    _keys = connection_keys()
    conn_parameters = {k: db_parameters[k] for k in _keys if k in db_parameters}
    with snowflake.connector.connect(**conn_parameters) as conn:
        conn._authenticator = "PAT_WITH_EXTERNAL_SESSION"
        conn._token = pat
        conn._external_session_id = external_session_id
        conn._rest._token = pat
        conn._rest._external_session_id = external_session_id
        yield conn


@pytest.fixture
def setup_with_connector_execution(connection):
    @contextmanager
    def _setup(sqls_to_enable: list[StrictStr], sqls_to_disable: list[StrictStr]):
        with connection.cursor() as cursor:
            for sql in sqls_to_enable:
                cursor.execute(sql)

            try:
                yield
            finally:
                for sql in sqls_to_disable:
                    cursor.execute(sql)

    return _setup


@pytest.fixture(scope="module")
def spark_connection(request) -> SnowflakeConnection:
    yield request.getfixturevalue(request.param)


@pytest.fixture(autouse=True)
# TODO: SNOW-1545034 make sure the role used here not hurt account
def use_accountadmin(connection, request):
    if "use_accountadmin" not in request.keywords:
        yield
        return

    with connection.cursor() as cursor, backup_role(cursor):
        try:
            cursor.execute("USE ROLE ACCOUNTADMIN")
        except Exception:
            pytest.xfail("Switch to role AccountAdmin failed")

        with suppress(Exception):
            yield


@pytest.fixture(scope="session")
def sf_cursor(connection, sf_connection_parameters, test_account, snowflake_version, snowflake_region):
    sf_conn = None
    try:
        # We can only run this if the test is against dev/reg env
        if is_prod_or_preprod(snowflake_version, snowflake_region) or sf_connection_parameters is None:
            yield None
        else:
            # return the cursor if we're already using the right account
            if test_account == SF_ACCOUNT:
                sf_conn = connection
            else:
                _keys = connection_keys()

                # switch to the snowflake account if connection config is available
                sf_conn = snowflake.connector.connect(
                    **{k: sf_connection_parameters[k] for k in _keys if k in sf_connection_parameters}
                )

            yield sf_conn.cursor()
    except snowflake.connector.Error:
        # We couldn't connect, yield None
        yield None
    finally:
        if sf_conn and test_account != SF_ACCOUNT:
            sf_conn.close()


@pytest.fixture(scope="session")
def sf_connection_parameters():
    # This does not work in notebooks
    if RUNNING_IN_NOTEBOOK or RUNNING_IN_STOREDPROC:
        yield None
    else:
        sf_connection_parameters = connection_config(connection_name="sf_account", error_if_not_exists=False)
        if sf_connection_parameters is None:
            yield None
        else:
            # Validate config; if the config isn't available, yield None
            if (
                "account" in sf_connection_parameters
                and sf_connection_parameters["account"].lower() == SF_ACCOUNT
                and "user" in sf_connection_parameters
                and "password" in sf_connection_parameters
            ):
                yield sf_connection_parameters
            else:
                yield None


@pytest.fixture(scope="session")
def require_sf(request, sf_cursor):
    if sf_cursor is None:
        pytest.skip("Skipping test because Snowflake account is required.")


@pytest.fixture
def set_internal_params(test_account, sf_cursor, require_sf):
    @contextmanager
    def _set_internal_params(parameters: dict[str, Any]):
        prefix = f"alter account {test_account}"

        # Warning: this will always unset the parameter to the default value,
        # only use this if that is ok.
        try:
            for k, v in parameters.items():
                sf_cursor.execute(f"{prefix} set {k} = {v};")
            yield
        finally:
            for k in parameters.keys():
                sf_cursor.execute(f"{prefix} unset {k};")

    return _set_internal_params


@pytest.fixture
def set_params(cursor):
    # Warning: this will always unset the parameters
    # to the default values without saving their previous settings;
    # only use this if that is ok.
    @contextmanager
    def _set_params(parameters: dict[str, Any], scope, target=""):
        prefix = f"alter {scope} {target}"

        try:
            for k, v in parameters.items():
                cursor.execute(f"{prefix} set {k} = {v};")
            yield
        finally:
            for k in parameters.keys():
                cursor.execute(f"{prefix} unset {k};")

    return _set_params


@pytest.fixture(autouse=True)
def internal_only(request, snowflake_version, snowflake_region):
    if "internal_only" in request.keywords and is_prod_or_preprod(snowflake_version, snowflake_region):
        pytest.skip("Test is skipped because it can only run in dev/reg environments.")


@pytest.fixture(autouse=True)
def prodlike_only(request, snowflake_version, snowflake_region):
    if "prodlike_only" in request.keywords and not is_prod_or_preprod(snowflake_version, snowflake_region):
        pytest.skip("Test is skipped because it can only run in prod or preprod environments.")


@pytest.fixture(autouse=True)
def verify_test_environment_pollution(cursor):
    # Get the function name
    _before_database = cursor.execute("SELECT CURRENT_DATABASE()").fetchone()[0]
    _before_schema = cursor.execute("SELECT CURRENT_SCHEMA()").fetchone()[0]
    _before_warehouse = cursor.execute("SELECT CURRENT_WAREHOUSE()").fetchone()[0]
    _before_role = cursor.execute("SELECT CURRENT_ROLE()").fetchone()[0]

    # Yield nothing
    try:
        yield
    finally:
        _after_database = cursor.execute("SELECT CURRENT_DATABASE()").fetchone()[0]
        _after_schema = cursor.execute("SELECT CURRENT_SCHEMA()").fetchone()[0]
        _after_warehouse = cursor.execute("SELECT CURRENT_WAREHOUSE()").fetchone()[0]
        _after_role = cursor.execute("SELECT CURRENT_ROLE()").fetchone()[0]
        assert _before_database == _after_database
        assert _before_schema == _after_schema
        assert _before_warehouse == _after_warehouse
        assert _before_role == _after_role



"""
Snowflake Notification Integration API.

The Snowflake Notification Integration API is a REST API that you can use to access, update, and perform certain actions on Notification Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.notification_integration._generated.models.notification_hook import (
    NotificationHook,
)


class NotificationQueueAzureEventGridOutbound(NotificationHook):
    """A model object representing the NotificationQueueAzureEventGridOutbound resource.

    Constructs an object of type NotificationQueueAzureEventGridOutbound with the provided properties.

    Parameters
    __________
    azure_event_grid_topic_endpoint : str, optional
        Event Grid topic endpoint to which Snowpipe pushes notifications.
    azure_tenant_id : str, optional
        ID of the Azure Active Directory tenant used for identity management.
    azure_consent_url : str, optional
        URL to the Microsoft permissions request page — **Read-only:** *any user-provided value will be ignored.*
    azure_multi_tenant_app_name : str, optional
        Name of the Snowflake client application created for your account — **Read-only:** *any user-provided value will be ignored.*
    """

    azure_event_grid_topic_endpoint: Optional[StrictStr] = None

    azure_tenant_id: Optional[StrictStr] = None

    azure_consent_url: Optional[StrictStr] = None

    azure_multi_tenant_app_name: Optional[StrictStr] = None

    __properties = ["type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NotificationQueueAzureEventGridOutbound:
        """Create an instance of NotificationQueueAzureEventGridOutbound from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "azure_consent_url",
                    "azure_multi_tenant_app_name",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["type"] = NotificationHook.get_child_model_discriminator_value("NotificationQueueAzureEventGridOutbound")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationQueueAzureEventGridOutbound:
        """Create an instance of NotificationQueueAzureEventGridOutbound from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return NotificationQueueAzureEventGridOutbound.model_validate(obj)

        _obj = NotificationQueueAzureEventGridOutbound.model_validate(
            {
                "azure_event_grid_topic_endpoint": obj.get("azure_event_grid_topic_endpoint"),
                "azure_tenant_id": obj.get("azure_tenant_id"),
                "azure_consent_url": obj.get("azure_consent_url"),
                "azure_multi_tenant_app_name": obj.get("azure_multi_tenant_app_name"),
            }
        )

        return _obj


class NotificationQueueAzureEventGridOutboundModel(NotificationHook):
    def __init__(
        self,  # optional properties
        azure_event_grid_topic_endpoint: Optional[str] = None,
        azure_tenant_id: Optional[str] = None,
        azure_consent_url: Optional[str] = None,
        azure_multi_tenant_app_name: Optional[str] = None,
    ):
        """A model object representing the NotificationQueueAzureEventGridOutbound resource.

        Constructs an object of type NotificationQueueAzureEventGridOutbound with the provided properties.

        Parameters
        __________
        azure_event_grid_topic_endpoint : str, optional
            Event Grid topic endpoint to which Snowpipe pushes notifications.
        azure_tenant_id : str, optional
            ID of the Azure Active Directory tenant used for identity management.
        azure_consent_url : str, optional
            URL to the Microsoft permissions request page.
        azure_multi_tenant_app_name : str, optional
            Name of the Snowflake client application created for your account.
        """
        super().__init__()
        self.azure_event_grid_topic_endpoint = azure_event_grid_topic_endpoint
        self.azure_tenant_id = azure_tenant_id
        self.azure_consent_url = azure_consent_url
        self.azure_multi_tenant_app_name = azure_multi_tenant_app_name

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return NotificationQueueAzureEventGridOutbound(
            azure_event_grid_topic_endpoint=self.azure_event_grid_topic_endpoint,
            azure_tenant_id=self.azure_tenant_id,
            azure_consent_url=self.azure_consent_url,
            azure_multi_tenant_app_name=self.azure_multi_tenant_app_name,
        )

    @classmethod
    def _from_model(cls, model) -> NotificationQueueAzureEventGridOutboundModel:
        return NotificationQueueAzureEventGridOutboundModel(
            azure_event_grid_topic_endpoint=model.azure_event_grid_topic_endpoint,
            azure_tenant_id=model.azure_tenant_id,
            azure_consent_url=model.azure_consent_url,
            azure_multi_tenant_app_name=model.azure_multi_tenant_app_name,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a NotificationQueueAzureEventGridOutbound.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the NotificationQueueAzureEventGridOutbound object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationQueueAzureEventGridOutboundModel:
        """Create an instance of NotificationQueueAzureEventGridOutbound from a dict.

        This method constructs a NotificationQueueAzureEventGridOutbound object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        NotificationQueueAzureEventGridOutbound
            A NotificationQueueAzureEventGridOutbound object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(NotificationQueueAzureEventGridOutbound.from_dict(obj))


NotificationQueueAzureEventGridOutbound._model_class = NotificationQueueAzureEventGridOutboundModel



"""
Cortex Agent API.

OpenAPI 3.0 specification for the Cortex Agent API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any

from pydantic import BaseModel, ConfigDict

from snowflake.core.cortex.lite_agent_service._generated.models.tool_tool_spec import ToolToolSpec, ToolToolSpecModel


class Tool(BaseModel):
    """A model object representing the Tool resource.

    Constructs an object of type Tool with the provided properties.

    Parameters
    __________
    tool_spec : ToolToolSpec
    """

    tool_spec: ToolToolSpec

    __properties = ["tool_spec"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Tool:
        """Create an instance of Tool from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of tool_spec
        if self.tool_spec:
            _dict["tool_spec"] = self.tool_spec.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Tool:
        """Create an instance of Tool from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Tool.model_validate(obj)

        _obj = Tool.model_validate(
            {
                "tool_spec": ToolToolSpec.from_dict(obj.get("tool_spec")) if obj.get("tool_spec") is not None else None,
            }
        )

        return _obj


class ToolModel:
    def __init__(
        self,
        tool_spec: ToolToolSpec,
        # optional properties
    ):
        """A model object representing the Tool resource.

        Constructs an object of type Tool with the provided properties.

        Parameters
        __________
        tool_spec : ToolToolSpec
        """
        self.tool_spec = tool_spec

    __properties = ["tool_spec"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Tool(
            tool_spec=self.tool_spec._to_model() if self.tool_spec is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> ToolModel:
        return ToolModel(
            tool_spec=ToolToolSpecModel._from_model(model.tool_spec) if model.tool_spec else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Tool.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Tool object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ToolModel:
        """Create an instance of Tool from a dict.

        This method constructs a Tool object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Tool
            A Tool object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Tool.from_dict(obj))


Tool._model_class = ToolModel
